#############################################################################
#                                                                           #  
# LIFE LIVED AND LEFT:                                                      #
# ESTIMATING AGE-SPECIFIC SURVIVAL IN STABLE POPULATIONS WITH UNKNOWN AGES  #
#                                                                           #
# James W. VAUPEL            jvaupel@sdu.dk                                 #
# Francisco VILLAVICENCIO    fvillavicencio@health.sdu.dk                   #
#                                                                           #
#                                                              October 2018 #
#############################################################################


# Clear workspace
rm(list = ls())


###########################################################
### SECTION 4.2. EXAMPLE WITH SIMULATED DATA            ###  
###########################################################

# Survival function to generate a stable population
#  Example from Preston et al. (2001, pp. 138-141)
lx <- c(1, .6, .4, .2, .05, 0)

# Probability of dying between ages x and x + 1
qx <- 1 - lx[-1] / lx[-length(lx)]

# Vector of ages
xages <- 0:(length(qx)-1)

# Constant growth rate
r <- 0.01

# Initial population that at exact time t-5 is age [0,1)
N0 <- 100000

# Generate a stable population at exact time t
N <- round(N0 * lx * exp(r*5:0))
# Population counts by age
N
# Total population
sum(N)

# Stable age structure of the population
cx <- N / sum(N)

# Create a vector with the ages of all the individuals alive at time t
pop <- rep(0:length(qx), times = N)

# Number of samples
nSample <- 100

# Size of each sample
nSize <- 1000

# Matrix to store Nhat values for each sample
NhatMat <- matrix(NA, ncol = length(qx), nrow = nSample)
NhatMat[, 1] <- nSize

# Draw 100 random samples of size 1000 from the population
for (i in 1:nSample) {
  # Sample of 1,000 individuals
  newPop <- sample(x = pop, size = nSize, replace = F)
  # Follow individuals of the sample until extinction
  for (j in 1:(length(qx)-1)) {
    # Age-specific probabilities of death of each individual
    probDeath <- qx[findInterval(newPop, xages)]
    # Randomly select the individuals dying in current iteration
    toDie <- which(runif(length(newPop)) < probDeath)
    # Update vector of survivors
    if (length(toDie) != 0) newPop <- newPop[-toDie]
    # Age one year
    newPop <- newPop + 1
    # Number of individuals still alive (Nhat)
    NhatMat[i, j+1] <- length(newPop)
  }
}

# Population at each age category from each sample - Equation (9*)
NxMat <- round(NhatMat[, -ncol(NhatMat)] - exp(-r)*NhatMat[, -1])
# Add last age group
NxMat <- cbind(NxMat, NhatMat[, ncol(NhatMat)])

# Age-specific SURVIVAL: Equation (10*)
lxMat <- NxMat / NxMat[, 1]
# Mean values across samples
lxHat <- round(apply(lxMat, 2, mean), 5)
# 95% empirical confidence intervals
lxLow <- round(apply(lxMat, 2, quantile, 0.025), 5)
lxUp <- round(apply(lxMat, 2, quantile, 0.975), 5)

# SURVIVORSHIP: Final table with results
Results1 <- data.frame(lxObs = lx, lxHat = c(lxHat, 0),
                       CI_2.5 = c(lxLow, 0), CI_97.5 = c(lxUp, 0))
Results1

# Stable AGE DISTRIBUTION: Equation (11*)
cxMat <- t(t(NxMat) * exp(-r*0:4)) / nSize
# Mean values across samples
cxHat <- round(apply(cxMat, 2, mean), 5)
# 95% empirical confidence intervals
cxLow <- round(apply(cxMat, 2, quantile, 0.025), 5)
cxUp <- round(apply(cxMat, 2, quantile, 0.975), 5)

# STABLE AGE STRUCTURE: Final table with results
Results2 <- data.frame(cxObs = cx, cxHat = c(cxHat, 0),
                       CI_2.5 = c(cxLow, 0), CI_97.5 = c(cxUp, 0))
Results2


###########################################################
### SECTION 5. APPLICATION: 1805 SWEDISH COHORT         ###  
###########################################################

# DATA: Population counts. Sweden, 1751-2017, 5-year intervals 
#  (Human Mortality Database, https://www.mortality.org/)
pop <- read.table("HMD-SwePop5x5.txt", skip = 2, header = T)
head(pop)

# DATA: Cohort life tables, Swedish females, 1751-1924, 5-year intervals
#  (Human Mortality Database, https://www.mortality.org/)
lifeTable <- read.table("HMD-SweFemLtCoh5x5.txt", skip = 2, header = T)
head(lifeTable)

# Years to be considered
years <- seq(1805, 1895, 5)

# Keep only data from the selected years
pop <- pop[pop$Year %in% years, c(1:3)]
head(pop)

# Total populations in 1805 and 1905
N0 <- sum(pop$Female[pop$Year == 1805])
N0
N1 <- sum(pop$Female[pop$Year == 1895])
N1

# Estimate growth rate for the whole period
r <- (log(N1) - log(N0)) / (1895 - 1805)
r

# Number of age categories in the data
ncat <- length(unique(pop$Age))

# Vector for Ntilde values
Ntilde <- N0

# Individuals alive in 1805 that are still alive at each subsequent 5-year period (Ntilde)
for (i in 2:length(years)) {
  Ntilde <- c(Ntilde, sum(pop$Female[pop$Year == years[i]][(i+1):ncat]))
}
Ntilde

# Apply Equation (9*) to estimate population counts at each age category
Nt <- Ntilde[-length(Ntilde)] - exp(-r)*Ntilde[-1]

# SURVIVAL: Apply Equation (10*) to estimate the life table of the 1805 cohort
lxHat <- Nt / Nt[1]
which(lxHat[-1] > lxHat[-length(lxHat)])
# Smooth data
x <- seq(0, 85, 5)
lxSmooth <- c(lxHat[1], predict(loess(lxHat[-1] ~ x[-1])))
which(lxSmooth[-1] > lxSmooth[-length(lxSmooth)])

# Life table of the 1805 cohort from the HMD
period <- paste(1805, 1805 + 4, sep = "-")
lxHMD <- lifeTable$lx[lifeTable$Year == period][-c(2, 20:24)]/100000

# Compare the two life tables
cbind(lxHMD, lxHat, lxSmooth)

# FIGURE 4 on the main manuscript
# pdf("Fig4-Surv1805cohort.pdf", width = 9, height = 5.5)
par(mar = c(4.3, 4.3, 1, 1), las = 1)
plot(x, lxHMD, type = 'l', lwd = 7, axes = F, ylim = c(0, 1), cex.lab = 1.5,
     ylab = 'survivorship', xlab = 'age', frame.plot = F)
lines(x, lxHat, col = "#932667FF", lwd = 7, lty = 2)
lines(x, lxSmooth, col = "#FCA50AFF", lwd = 7)
axis(1, at = seq(0, 85, 5), cex.axis = 1.3)
axis(2, at = seq(0, 1, 0.2), cex.axis = 1.3)
legend("topright", lwd = 6, col = c(1, "#932667FF", "#FCA50AFF"), lty = c(1, 2, 1),
       legend = c("HMD", "estimated", "estimated\n(smoothed)"), 
       bty = 'n', cex = 1.5, seg.len = 3.5, y.intersp = 1.3)
# dev.off()
